/* board.c - Board-specific hooks */

/*
 * Copyright (c) 2017 Intel Corporation
 * Additional Copyright (c) 2018 Espressif Systems (Shanghai) PTE LTD
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdio.h>

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"

#include "driver/uart.h"
#include "esp_log.h"

#include "esp_ble_mesh_provisioning_api.h"
#include "board.h"

#define TAG "BOARD"

#define MESH_UART_NUM UART_NUM_1
#define MESH_UART    (&UART1)

#define UART_BUF_SIZE 128

#define UART1_TX_PIN  GPIO_NUM_16
#define UART1_RX_PIN  GPIO_NUM_17

extern uint16_t remote_addr;
esp_ble_mesh_client_t onoff_client;
esp_ble_mesh_model_t * vnd_cli_model;
uint16_t node_app_idx = ESP_BLE_MESH_KEY_UNUSED;
struct _led_state led_state[3] = {
    { LED_OFF, LED_OFF, LED_R, "red"   },
    { LED_OFF, LED_OFF, LED_G, "green" },
    { LED_OFF, LED_OFF, LED_B, "blue"  },
};

void board_output_number(esp_ble_mesh_output_action_t action, uint32_t number)
{
    ESP_LOGI(TAG, "Board output number %d", number);
}

void board_prov_complete(void)
{
    board_led_operation(LED_G, LED_OFF);
}

void board_led_operation(uint8_t pin, uint8_t onoff)
{
    for (int i = 0; i < ARRAY_SIZE(led_state); i++) {
        if (led_state[i].pin != pin) {
            continue;
        }
        if (onoff == led_state[i].previous) {
            ESP_LOGW(TAG, "led %s is already %s",
                     led_state[i].name, (onoff ? "on" : "off"));
            return;
        }
        gpio_set_level(pin, onoff);
        led_state[i].previous = onoff;
        return;
    }
    ESP_LOGE(TAG, "LED is not found!");
}
void onoff_client_send_massage(uint16_t addr,bool state)
{
    int err = 0;
    esp_ble_mesh_client_common_param_t common = {0};
    esp_ble_mesh_generic_client_set_state_t set_state = {0};

    common.opcode = ESP_BLE_MESH_MODEL_OP_GEN_ONOFF_SET;
    common.model = onoff_client.model;
    common.ctx.net_idx = 0x0;
    // common.ctx.app_idx = 0x0; 
    common.ctx.app_idx = node_app_idx; 
    common.ctx.addr = addr;
    common.ctx.send_ttl = 0x3;
    common.ctx.send_rel = false;
    common.msg_timeout = 0x0;
    common.msg_role = ROLE_NODE;
    set_state.onoff_set.op_en = false;
    // set_state.onoff_set.onoff = state;
    set_state.onoff_set.onoff = 0x1;
    set_state.onoff_set.tid = 0x0;
    printf("%d \n",node_app_idx);
    err = esp_ble_mesh_generic_client_set_state(&common, &set_state);
    if (err != ESP_OK) {
        ESP_LOGE(TAG, "%s: Generic OnOff Set Unack failed", __func__);
        return;
    }
}
void vender_client_send_message(uint16_t addr)
{
    struct net_buf_simple *msg = NULL;
    esp_err_t err;

    msg = bt_mesh_alloc_buf(20);
    if (!msg) {
        return ESP_FAIL;
    }
    net_buf_simple_init(msg, 0);
//    net_buf_simple_add_le16(msg, 0x00);
//    net_buf_simple_add_le16(msg, 0x03);
//    net_buf_simple_add_le16(msg, 0x09);
//    net_buf_simple_add_le16(msg, 0x09);

    if (led_state[0].previous != LED_OFF)
    {
        net_buf_simple_add_mem(msg, "VEND OF\r", sizeof("VEND OF\r"));
        board_led_operation(LED_R, LED_OFF);
    }
    else
    {
        net_buf_simple_add_mem(msg, "VEND ON\r", sizeof("VEND ON\r"));
        board_led_operation(LED_R, LED_ON);
    }

    esp_ble_mesh_msg_ctx_t ctx = {
        .net_idx  = 0x0,
        .app_idx  = 0x0,
        .addr     = addr,
        .send_rel = false,
        .send_ttl = 0,
    };
    err = esp_ble_mesh_client_model_send_msg(vnd_cli_model, &ctx,
        ESP_BLE_MESH_VND_MODEL_OP_FAST_PROV_INFO_SET,
        msg->len, msg->data, 500, false, ROLE_NODE);
}
static void board_uart_task(void *p)
{
    uint8_t *data = calloc(1, UART_BUF_SIZE);
    uint32_t input;

    while (1) {
        int len = uart_read_bytes(MESH_UART_NUM, data, UART_BUF_SIZE, 100 / portTICK_RATE_MS);
        if (len > 0) {
            input = strtoul((const char *)data, NULL, 16);
            remote_addr = input & 0xFFFF;
            ESP_LOGI(TAG, "%s: input 0x%08x, remote_addr 0x%04x, '%s' ", __func__, input, remote_addr, data);
            memset(data, 0, UART_BUF_SIZE);
            // onoff_client_send_massage(remote_addr,0x1);

            vender_client_send_message(remote_addr);
        }
        else if (gpio_get_level(SWITCH_PIN) == 0)
        {
            ESP_LOGI(TAG, "%s: button remote_addr 0x%04x, '%s' ", __func__, remote_addr, data);
            vender_client_send_message(remote_addr);
            vTaskDelay(1000 / portTICK_RATE_MS);
        }
    }

    vTaskDelete(NULL);
}

static void board_led_init(void)
{
    for (int i = 0; i < ARRAY_SIZE(led_state); i++) {
        gpio_pad_select_gpio(led_state[i].pin);
        gpio_set_direction(led_state[i].pin, GPIO_MODE_OUTPUT);
        gpio_set_level(led_state[i].pin, LED_OFF);
        led_state[i].previous = LED_OFF;
    }
}

static void board_uart_init(void)
{
    uart_config_t uart_config = {
        .baud_rate = 115200,
        .data_bits = UART_DATA_8_BITS,
        .parity = UART_PARITY_DISABLE,
        .stop_bits = UART_STOP_BITS_1,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE
    };
    uart_param_config(MESH_UART_NUM, &uart_config);
    uart_set_pin(MESH_UART_NUM, UART1_TX_PIN, UART1_RX_PIN, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE);
    uart_driver_install(MESH_UART_NUM, UART_BUF_SIZE * 2, 0, 0, NULL, 0);
}

void board_init(void)
{
    gpio_pad_select_gpio(SWITCH_PIN);
    gpio_set_direction(SWITCH_PIN, GPIO_MODE_INPUT);

    board_led_init();
    board_uart_init();
    xTaskCreate(board_uart_task, "board_uart_task", 2048, NULL, 5, NULL);
}
